
/******************************************************************************
* MODULE     : declare.cc
* DESCRIPTION: Declaring new global or local variables
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
namespace mmx {

value
overload (value v1, value v2) {
  if (!is_func (v1->type, ID_AND))
    return value (list (MMX_AND, v2->val, v1->val),
		  list (ID_AND, v2->type, v1->type));
  else
    return value (cons (MMX_AND, v2->val, cdr (as_list (v1->val))),
		  cons (ID_AND, v2->type, cdr (as_list (v1->type))));
}

void
evaluator_rep::declare (symbol var, value val) {
  // globally declare the symbol var as val
  // may also overload var

  if (env->contains (var)) {
    if (!is_value (env[var])) {
      error (var, "Invalid overloading");
      return;
    }
    env (var)= overload (as_value (env[var]), val);
  }
  else env (var)= val;
}

void
evaluator_rep::declare (list decls) {
  // locally declare a list of variables
  // each item in "decls" is a pair (var, val),
  // where var is a symbol and val a typed value

  while (!nil (decls)) {
    env->write (car (decls));
    decls= cdr (decls);
  }
}

} // namespace mmx
